/*
 * Query NSS netgroup information from python
 *
 * @author Joel Heenan 3/12/2008
 *
 * Licensed under the GPL v2 license which should be attached
 *
 * 4/12/2008 - JH - Fixed Py_True/Py_False so they are INCREF'd
 *                  (thanks Jacobs of bioinformed.com)
 */

#include <Python.h>

static PyObject *
getgroup(PyObject *self, PyObject *args) {
    const char *netgroup;
    char *p[3];

    if(!PyArg_ParseTuple(args, "s", &netgroup))
        return NULL;
    
    setnetgrent(netgroup);
    PyObject* theList = PyList_New(0);
    
    while(getnetgrent(p, p+1, p+2)) {
        PyObject* tuple = Py_BuildValue("sss",p[0],p[1],p[2]);
        PyList_Append(theList,tuple);
        Py_DECREF(tuple);
    }

    endnetgrent();

    return theList;
}

PyDoc_STRVAR(getgroup_doc,
"getgroup(netgroupName)\n"
"\n"
"Retrieve a netgroup using NSS routines\n"
"Returns a list of matching (host,user,domain) tuples"
);

static PyObject *
innetgr_py(PyObject *self, PyObject *args, PyObject *keywds) {
    char *netgroup=NULL, *host=NULL, *user=NULL, *domain=NULL;

    static char *kwlist[] = { "netgroup", "host", "user", "domain", NULL};

    if(!PyArg_ParseTupleAndKeywords(args,keywds,"s|sss",kwlist,&netgroup,&host,&user,&domain)) 
        return NULL;

    if(innetgr(netgroup,host,user,domain)) {
        Py_INCREF(Py_True);
        return Py_True;
    } else {
        Py_INCREF(Py_False);
        return Py_False;
    }
}

PyDoc_STRVAR(innetgr_doc,
"innetgr(netgroup,host=host,user=user,domain=domain) -> bool\n"
"\n"
"Ask whether a host/user/domain tuple is part of a netgroup\n"
"If no host,user or domain is passed then it returns true if the netgroup exists\n"
);

PyMethodDef methods[] = {
    {"getgroup", getgroup, METH_VARARGS, getgroup_doc},
    {"innetgr", (PyCFunction)innetgr_py, METH_VARARGS|METH_KEYWORDS, innetgr_doc},
    {NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
initnetgroup() {
    (void) Py_InitModule3("netgroup", methods, "This module is for querying NSS netgroups");
}
